/*
 * Main program entry point - read the command line options, then perform
 * the appropriate actions.
 *
 * Copyright 2004 Andrew Wood, distributed under the Artistic License.
 */

#include "config.h"
#include "options.h"
#include "message.h"
#include "spam.h"
#include "database.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#ifdef HAVE_MCHECK_H
#include <mcheck.h>
#endif


/*
 * Process command-line arguments and set option flags, then call functions
 * to initialise, and finally enter the main loop.
 */
int main(int argc, char **argv)
{
	char buffer[1024];
	char filename[1024];
	opts_t options;
	int retcode = 0;
	int needwrite = 0;
	int got, sent, totsent;
	double score;
	qdb_t dbr1, dbr2, dbw;
	char *home;
	msg_t msg = NULL;
	int fd = -1;

#ifdef HAVE_MCHECK_H
	if (getenv("MALLOC_TRACE"))
		mtrace();
#endif

#ifdef ENABLE_NLS
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
#endif

	options = parse_options(argc, argv);
	if (!options)
		return 1;

	if (options->do_nothing) {
		options->destructor(options);
		return 0;
	}

	/*
	 * If we're not dumping, restoring, token listing, pruning,
	 * training, benchmarking, or merging, we need to read a message
	 * from standard input before proceeding.
	 */
	if (!options->dump && !options->restore
	    && !options->train && !options->benchmark
	    && !options->tokens && !options->prune && !options->merge) {
		msg = msg_parse(options);
		if (msg == NULL) {
			options->destructor(options);
			return 1;
		} else if (msg->content == NULL) {
			msg_dump(msg);
			msg_free(msg);
			while (!feof(stdin)) {
				got =
				    fread(buffer, 1, sizeof(buffer),
					  stdin);
				totsent = 0;
				if (got > 0) {
					sent = fwrite(buffer + totsent, 1,
						      got, stdout);
					if (sent <= 0)
						break;
					totsent += sent;
					got -= sent;
				} else {
					break;
				}
			}
			options->destructor(options);
			return 0;
		}
	}

	/*
	 * If we're just tokenising, just do that with the message we parsed
	 * above, then exit.
	 */
	if (options->tokens) {
		msg = msg_parse(options);
		if (msg == NULL) {
			options->destructor(options);
			return 1;
		}
		retcode = spam_dumptokens(options, msg);
		msg_free(msg);
		return retcode;
	}

	/*
	 * From this point on, we're going to need databases; if none was
	 * specified on the command line, we open both /var/lib/qsfdb and
	 * ~/.qsfdb.
	 */

	dbr1 = NULL;			    /* first database to read from */
	dbr2 = NULL;			    /* second database to write from */
	dbw = NULL;			    /* database to write to (=dbr1 or dbr2 or NULL) */

	if (options->dump || options->restore || options->train
	    || options->mark_spam || options->mark_nonspam
	    || options->prune || options->merge)
		needwrite = 1;

	if (options->benchmark) {

#ifdef P_tmpdir
		strcpy(filename, P_tmpdir "/qsfXXXXXX");
#else
		strcpy(filename, "/tmp/qsfXXXXXX");
#endif

#ifdef HAVE_MKSTEMP
		fd = mkstemp(filename);
#else
		tmpnam(filename);
		fd = open(filename, O_RDWR | O_CREAT | O_EXCL,
			  S_IRUSR | S_IWUSR);
#endif
		if (fd < 0) {
			fprintf(stderr, "%s: %s: %s: %s\n",
				options->program_name,
				filename,
				_("failed to create temporary file"),
				strerror(errno));
			retcode = 1;
		} else {
			chmod(filename, S_IRUSR | S_IWUSR);
			dbr1 = qdb_open(filename, QDB_READWRITE);
			dbw = dbr1;
			if (dbr1 == NULL) {
				fprintf(stderr, "%s: %s: %s: %s\n",
					options->program_name,
					filename,
					_("failed to open database"),
					qdb_error());
				retcode = 1;
			}
			close(fd);
			remove(filename);
		}

	} else if (options->database == NULL) {

		if (options->globaldb) {
			sprintf(buffer, "%.*s", (int) (sizeof(buffer) - 1),
				options->globaldb);
		} else {
			sprintf(buffer, "%s", "/var/lib/" PACKAGE "db");
		}
		if (needwrite) {
			if (options->restore) {
				dbr1 = qdb_open(buffer, QDB_NEW);
			} else {
				dbr1 = qdb_open(buffer, QDB_READWRITE);
			}
		}
		if (dbr1 == NULL) {
			dbr1 = qdb_open(buffer, QDB_READONLY);
		} else {
			dbw = dbr1;
		}

		home = getenv("HOME");
		if (home == NULL)
			home = "/";

		if (strlen(home) > (sizeof(buffer) - 64))
			home = "/";

		sprintf(buffer, "%s/.%sdb", home, PACKAGE);

		if (needwrite) {
			if (options->restore) {
				dbr2 = qdb_open(buffer, QDB_NEW);
			} else {
				dbr2 = qdb_open(buffer, QDB_READWRITE);
			}
		}
		if (dbr2 == NULL) {
			dbr2 = qdb_open(buffer, QDB_READONLY);
		} else if (dbw == NULL) {
			dbw = dbr2;
		}

	} else {

		dbr1 = qdb_open(options->database, QDB_READWRITE);
		if (dbr1 == NULL) {
			dbr1 = qdb_open(options->database, QDB_READONLY);
		} else {
			dbw = dbr1;
		}
		if (dbr1 == NULL) {
			fprintf(stderr, "%s: %s: %s: %s\n",
				options->program_name,
				options->database,
				_("failed to open database"), qdb_error());
			retcode = 1;
		}
		if (options->globaldb) {
			dbr2 = qdb_open(options->globaldb, QDB_READONLY);
			if (dbr2 == NULL) {
				fprintf(stderr, "%s: %s: %s: %s\n",
					options->program_name,
					options->globaldb,
					_("failed to open database"),
					qdb_error());
				retcode = 1;
			}
		}
	}

	options->dbr1 = dbr1;
	options->dbr2 = dbr2;
	options->dbw = dbw;

	if (retcode) {
		options->destructor(options);
		qdb_close(dbr1);
		qdb_close(dbr2);
		return retcode;
	} else if (options->dump) {
		retcode = spam_db_dump(options);
		options->destructor(options);
		qdb_close(dbr1);
		qdb_close(dbr2);
		return retcode;
	} else if (options->restore) {
		retcode = spam_db_restore(options);
		options->destructor(options);
		qdb_close(dbr1);
		qdb_close(dbr2);
		return retcode;
	} else if (options->prune) {
		retcode = spam_db_prune(options);
		if (retcode == 0) {
			printf("%s", _("Optimising database..."));
			qdb_optimise(dbw);
			printf(" %s\n", _("done"));
		}
		options->destructor(options);
		qdb_close(dbr1);
		qdb_close(dbr2);
		return retcode;
	} else if (options->train) {
		retcode = spam_train(options);
		options->destructor(options);
		qdb_close(dbr1);
		qdb_close(dbr2);
		return retcode;
	} else if (options->benchmark) {
		retcode = spam_benchmark(options);
		options->destructor(options);
		qdb_close(dbr1);
		remove(filename);
		return retcode;
	} else if (options->merge) {
		retcode = spam_db_merge(options);
		options->destructor(options);
		/*
		 * Note we are closing options->dbr1 etc instead of our dbr1
		 * etc because spam_db_merge() shuffles them around.
		 */
		qdb_close(options->dbr1);
		qdb_close(options->dbr2);
		return retcode;
	}

	if (options->mark_spam) {
		retcode = spam_update(options, msg, SPAM);
		msg_free(msg);
	} else if (options->mark_nonspam) {
		retcode = spam_update(options, msg, NONSPAM);
		msg_free(msg);
	} else {
		score = spam_check(options, msg);
		if (!options->no_header)
			msg_spamheader(msg, score);
		if (options->add_rating) {
			msg_spamratingheader(msg, score);
			if (options->no_filter) {
				double spamscore, scaledscore;

				spamscore = score;
				if (spamscore < 0)
					spamscore += 0.01;
				spamscore += 0.90;
				scaledscore = spamscore * 100.0;
				printf("%d\n", (int) scaledscore);
			}
		}
		if (score > 0) {
			if (options->modify_subject)
				msg_spamsubject(msg,
						options->subject_marker);
			if (!options->no_filter) {
				msg_dump(msg);
				retcode = 0;
			} else {
				retcode = 1;
			}
		} else {
			if (!options->no_filter)
				msg_dump(msg);
			retcode = 0;
		}
		msg_free(msg);
	}

	options->destructor(options);

	qdb_close(dbr1);
	qdb_close(dbr2);

	return retcode;
}

/* EOF */
