/*
 * Parse command-line options.
 *
 * Copyright 2004 Andrew Wood, distributed under the Artistic License.
 */

#include "config.h"
#include "options.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <unistd.h>

#ifndef HAVE_GETOPT

int minigetopt(int, char **, char *);
extern char *minioptarg;
extern int minioptind, miniopterr, minioptopt;

#define getopt minigetopt
#define optarg minioptarg
#define optind minioptind
#define opterr miniopterr
#define optopt minioptopt

#endif				/* !HAVE_GETOPT */


void display_help(void);
void display_license(void);
void display_version(void);


/*
 * Free an opts_t object.
 */
void parse_options__destroy(opts_t object)
{
	if (!object)
		return;
	if (object->argv)
		free(object->argv);
	free(object);
}


/*
 * Parse the given command-line arguments into an opts_t object, handling
 * "help", "license" and "version" options internally.
 *
 * Returns an opts_t, or 0 on error.
 *
 * Note that the contents of *argv[] (i.e. the command line parameters)
 * aren't copied anywhere, just the pointers are copied, so make sure the
 * command line data isn't overwritten or argv[1] free()d or whatever.
 */
opts_t parse_options(int argc, char **argv)
{
#ifdef HAVE_GETOPT_LONG
	struct option long_options[] = {
		{"help", 0, 0, 'h'},
		{"license", 0, 0, 'l'},
		{"version", 0, 0, 'V'},
		{"subject", 0, 0, 's'},
		{"subject-marker", 1, 0, 'S'},
		{"no-header", 0, 0, 'n'},
		{"rate", 0, 0, 'r'},
		{"rating", 0, 0, 'r'},
		{"add-rating", 0, 0, 'r'},
		{"add-tokens", 0, 0, 'o'},
		{"test", 0, 0, 't'},
		{"allowlist", 0, 0, 'a'},
		{"allow-list", 0, 0, 'a'},
		{"mark-spam", 0, 0, 'm'},
		{"mark-nonspam", 0, 0, 'M'},
		{"mark-non-spam", 0, 0, 'M'},
		{"database", 1, 0, 'd'},
		{"global", 1, 0, 'g'},
		{"weight", 1, 0, 'w'},
		{"train", 0, 0, 'T'},
		{"prune", 0, 0, 'p'},
		{"trim", 0, 0, 'p'},
		{"dump", 0, 0, 'D'},
		{"restore", 0, 0, 'R'},
		{"tokens", 0, 0, 'O'},
		{"benchmark", 0, 0, 'B'},
		{"merge", 1, 0, 'E'},
		{0, 0, 0, 0}
	};
	int option_index = 0;
#endif
	char *short_options = "hlVsS:nrtoamMd:g:w:TpDROBE:";
	int c, numopts;
	opts_t options;

	options = calloc(1, sizeof(*options));
	if (!options) {
		fprintf(stderr,
			_("%s: option structure allocation failed (%s)"),
			argv[0], strerror(errno));
		fprintf(stderr, "\n");
		return 0;
	}

	options->program_name = argv[0];

	options->destructor = parse_options__destroy;

	options->argc = 0;
	options->argv = calloc(argc + 1, sizeof(char *));
	if (!options->argv) {
		fprintf(stderr,
			_
			("%s: option structure argv allocation failed (%s)"),
			argv[0], strerror(errno));
		fprintf(stderr, "\n");
		options->destructor(options);
		return 0;
	}

	numopts = 0;

	do {
#ifdef HAVE_GETOPT_LONG
		c = getopt_long(argc, argv,
				short_options, long_options,
				&option_index);
#else
		c = getopt(argc, argv, short_options);
#endif

		if (c < 0)
			continue;

		switch (c) {
		case 'h':
			display_help();
			options->do_nothing = 1;
			return options;
			break;
		case 'l':
			display_license();
			options->do_nothing = 1;
			return options;
			break;
		case 'V':
			display_version();
			options->do_nothing = 1;
			return options;
			break;
		case 's':
			options->modify_subject = 1;
			numopts++;
			break;
		case 'S':
			options->modify_subject = 1;
			options->subject_marker = optarg;
			numopts++;
			break;
		case 'n':
			options->no_header = 1;
			numopts++;
			break;
		case 'r':
			options->add_rating = 1;
			numopts++;
			break;
		case 't':
			options->no_filter = 1;
			numopts++;
			break;
		case 'o':
			fprintf(stderr, "%s: %s\n", argv[0],
				_
				("warning: option \"-o\" / \"--add-tokens\" is no longer available"));
			break;
		case 'a':
			options->allowlist = 1;
			numopts++;
			break;
		case 'm':
			options->mark_spam = 1;
			numopts++;
			break;
		case 'M':
			options->mark_nonspam = 1;
			numopts++;
			break;
		case 'd':
			options->database = optarg;
			numopts++;
			break;
		case 'g':
			options->globaldb = optarg;
			numopts++;
			break;
		case 'w':
			options->weight = atoi(optarg);
			numopts++;
			break;
		case 'T':
			options->train = 1;
			numopts++;
			break;
		case 'p':
			options->prune = 1;
			numopts++;
			break;
		case 'D':
			options->dump = 1;
			numopts++;
			break;
		case 'R':
			options->restore = 1;
			numopts++;
			break;
		case 'O':
			options->tokens = 1;
			numopts++;
			break;
		case 'B':
			options->benchmark = 1;
			numopts++;
			break;
		case 'E':
			options->merge = 1;
			options->mergefrom = optarg;
			numopts++;
			break;
		default:
#ifdef HAVE_GETOPT_LONG
			fprintf(stderr,
				_("Try `%s --help' for more information."),
				argv[0]);
#else
			fprintf(stderr,
				_("Try `%s -h' for more information."),
				argv[0]);
#endif
			fprintf(stderr, "\n");
			options->destructor(options);
			return 0;
			break;
		}

	} while (c != -1);

	if (options->weight < 1) {
		options->weight = 1;
	} else if (options->weight > 8) {
		options->weight = 8;
	}

	while (optind < argc) {
		options->argv[options->argc++] = argv[optind++];
	}

	if (options->train && options->argc != 2) {
		fprintf(stderr, "%s: %s\n", argv[0],
			_
			("train requires two arguments, spam and non-spam"));
		options->destructor(options);
		return 0;
	}

	if (options->benchmark && options->argc != 2) {
		fprintf(stderr, "%s: %s\n", argv[0],
			_
			("benchmark requires two arguments, spam and non-spam"));
		options->destructor(options);
		return 0;
	}

	return options;
}

/* EOF */
